<?php

namespace PAR\Controller;

use PAR\Controller\PARCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use PAR\Service\BusinessPartnerService;
use PAR\Traits\TResponse;

class BusinessPartnerController extends PARCrud
{

    use TResponse;

    protected $businessPartnerService;

    public function __construct($crudService, $environment, BusinessPartnerService $businessPartnerService)
    {
        parent::__construct($crudService, $environment);
        $this->businessPartnerService = $businessPartnerService;
    }

    public function getBusinessPartner()
    {
        try {
            $data = $this->businessPartnerService->getBusinessPartner();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getOperator()
    {
        try {
            $data = $this->businessPartnerService->getOperator();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getPartnerClassification()
    {
        try {
            $data = $this->businessPartnerService->getPartnerClassification();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getTypeOfPartner()
    {
        try {
            $data = $this->businessPartnerService->getTypeOfPartner();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getMarketSegment()
    {
        try {
            $data = $this->businessPartnerService->getMarketSegment();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getBranchOfActivity()
    {
        try {
            $data = $this->businessPartnerService->getBranchOfActivity();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getTypeOfActivity()
    {
        try {
            $data = $this->businessPartnerService->getTypeOfActivity();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getCountry()
    {
        try {
            $data = $this->businessPartnerService->getCountry();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getState(Request $request)
    {
        try {
            $data = $this->businessPartnerService->getState($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getCity(Request $request)
    {
        try {
            $data = $this->businessPartnerService->getCity($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getDistrict(Request $request)
    {
        try {
            $data = $this->businessPartnerService->getDistrict($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findCodeDistrict(DTO\Request $request, DTO\Response $response)
    {
        try {
            $codeFound = $this->businessPartnerService->findCodeDistrict($request->getParameters());
            $dtsVal = new DataSet('codeFound', $codeFound);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getSpecificBusinessPartner(DTO\Request $request, DTO\Response $response)
    {
        try {
            $partnerFound = $this->businessPartnerService->getSpecificBusinessPartner($request->getParameters());
            $dtsVal = new DataSet('partnerFound', $partnerFound);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function insertNewBusinessPartner(DTO\Request $request, DTO\Response $response)
    {
        try {
            $partner = $this->businessPartnerService->insertNewBusinessPartner($request->getParameters());
            $dtsVal = new DataSet('partner', [$partner]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function updateSpecificBusinessPartner(DTO\Request $request, DTO\Response $response)
    {
        try {
            $partnerFound = $this->businessPartnerService->updateSpecificBusinessPartner($request->getParameters());
            $dtsVal = new DataSet('partnerFound', [$partnerFound]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getBusinessPartnerAddress(Request $request)
    {
        try {
            $data = $this->businessPartnerService->getBusinessPartnerAddress($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getSpecificAddress(DTO\Request $request, DTO\Response $response)
    {
        try {
            $addressFound = $this->businessPartnerService->getSpecificAddress($request->getParameters());
            $dtsVal = new DataSet('addressFound', $addressFound);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function insertNewAddress(DTO\Request $request, DTO\Response $response)
    {
        try {
            $address = $this->businessPartnerService->insertNewAddress($request->getParameters());
            $dtsVal = new DataSet('address', [$address]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteAddress(DTO\Request $request)
    {
        try {
            $codigos = $request->getParameters()["NRENDERECOPARC"];
            $this->businessPartnerService->deleteAddress($codigos);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function updateAddress(DTO\Request $request, DTO\Response $response)
    {
        try {
            $address = $this->businessPartnerService->updateAddress($request->getParameters());
            $dtsVal = new DataSet('address', [$address]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    // --------- FUNÇÕES DE COMUNICAÇÃO -----------------------
    public function getWayOfCommunication(Request $request)
    {
        try {
            $data = $this->businessPartnerService->getWayOfCommunication($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getListWaysOfCommunication()
    {
        try {
            $data = $this->businessPartnerService->getListWaysOfCommunication();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getSpecificWayOfCommunication(DTO\Request $request, DTO\Response $response)
    {
        try {
            $communicFound = $this->businessPartnerService->getSpecificWayOfCommunication($request->getParameters());
            $dtsVal = new DataSet('communicFound', $communicFound);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function insertNewCommunicationData(DTO\Request $request, DTO\Response $response)
    {
        try {
            $communic = $this->businessPartnerService->insertNewCommunicationData($request->getParameters());
            $dtsVal = new DataSet('communic', [$communic]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteWayOfCommunication(DTO\Request $request, DTO\Response $response)
    {
        try {
            $communic = $this->businessPartnerService->deleteWayOfCommunication($request->getParameters());
            $dtsVal = new DataSet('communic', [$communic]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function updateWayOfCommunication(DTO\Request $request, DTO\Response $response)
    {
        try {
            $communic = $this->businessPartnerService->updateWayOfCommunication($request->getParameters());
            $dtsVal = new DataSet('communic', [$communic]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    // --------- FUNÇÕES DE INTERLOCUTORES -----------------------
    public function getSectors()
    {
        try {
            $data = $this->businessPartnerService->getSectors();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getPositionsOfInterlocutors()
    {
        try {
            $data = $this->businessPartnerService->getPositionsOfInterlocutors();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getInterlocutors(Request $request)
    {
        try {
            $data = $this->businessPartnerService->getInterlocutors($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getSpecificInterlocutor(Request $request, Response $response)
    {
        try {
            $interFound = $this->businessPartnerService->getSpecificInterlocutor($request->getParameters());
            $dtsVal = new DataSet('interFound', $interFound);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function insertNewInterlocutor(Request $request, Response $response)
    {
        try {
            $interlocutor = $this->businessPartnerService->insertNewInterlocutor($request->getParameters());
            $dtsVal = new DataSet('interlocutor', [$interlocutor]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteInterlocutor(Request $request, Response $response)
    {
        try {
            $interlocutor = $this->businessPartnerService->deleteInterlocutor($request->getParameters());
            $dtsVal = new DataSet('interlocutor', [$interlocutor]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function updateInterlocutor(Request $request, Response $response)
    {
        try {
            $interlocutor = $this->businessPartnerService->updateInterlocutor($request->getParameters());
            $dtsVal = new DataSet('interlocutor', [$interlocutor]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    // --------- FUNÇÕES DE DADOS BANCÁRIOS -----------------------
    public function getBankDetails(Request $request)
    {
        try {
            $data = $this->businessPartnerService->getBankDetails($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getBanks()
    {
        try {
            $data = $this->businessPartnerService->getBanks();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getAgencies(Request $request)
    {
        try {
            $data = $this->businessPartnerService->getAgencies($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getCheckingAccount(Request $request)
    {
        try {
            $data = $this->businessPartnerService->getCheckingAccount($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getTypeOfCheckingAccount()
    {
        try {
            $data = $this->businessPartnerService->getTypeOfCheckingAccount();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getSpecificBankDetails(Request $request, Response $response)
    {
        try {
            $detailsFound = $this->businessPartnerService->getSpecificBankDetails($request->getParameters());
            $dtsVal = new DataSet('detailsFound', $detailsFound);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function insertNewBankDetails(Request $request, Response $response)
    {
        try {
            $bank = $this->businessPartnerService->insertNewBankDetails($request->getParameters());
            $dtsVal = new DataSet('bank', [$bank]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteBankDetails(Request $request, Response $response)
    {
        try {
            $bank = $this->businessPartnerService->deleteBankDetails($request->getParameters());
            $dtsVal = new DataSet('bank', [$bank]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function updateBankDetails(Request $request, Response $response)
    {
        try {
            $bank = $this->businessPartnerService->updateBankDetails($request->getParameters());
            $dtsVal = new DataSet('bank', [$bank]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getBranches()
    {
        try {
            $data = $this->businessPartnerService->getBranches();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getInvoiceSeries()
    {
        try {
            $data = $this->businessPartnerService->getInvoiceSeries();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getBillingParameter(Request $request, Response $response)
    {
        try {
            $parameter = $this->businessPartnerService->getBillingParameter($request->getParameters());
            $dtsVal = new DataSet('parameter', $parameter);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function billingParameter(Request $request, Response $response)
    {
        try {
            $parameter = $this->businessPartnerService->billingParameter($request->getParameters());
            $dtsVal = new DataSet('parameter', [$parameter]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function sendImage(DTO\Request $request, DTO\Response $response)
    {
        try {
            $image = $this->businessPartnerService->sendImage($request->getParameters());
            $dtsVal = new DataSet('image', [$image]);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteImage(DTO\Request $request)
    {
        try {
            $this->businessPartnerService->deleteImage($request->getParameters());
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getImagesLogotype(DTO\Request $request, DTO\Response $response)
    {
        try {
            $image = $this->businessPartnerService->getImagesLogotype($request->getParameters());
            $dtsVal = new DataSet('image', $image);
            $response->addDataSet($dtsVal);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getBusinessPartnerStandardFinancialIndices(Request $request)
    {
        try {
            $data = $this->businessPartnerService->getBusinessPartnerStandardFinancialIndices($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function salvarIndiceFinanceiroPadraoParceiro(Request $request)
    {
        try {
            $params = $request->getParameters();
            $nrindicespadrparc = isset($params["NRINDICESPADRPARC"]) ? $params["NRINDICESPADRPARC"] : null;
            $nrparcnegocio     = $params["NRPARCNEGOCIO"];
            $cdtpindice        = $params["CDTPINDICE"];
            $idpervalor        = $params["IDPERVALOR"];
            $cdindice          = $params["CDINDICE"];
            $nrpercentindice   = $params["NRPERCENTINDICE"];

            $this->businessPartnerService->salvarIndiceFinanceiroPadraoParceiro($nrindicespadrparc, $nrparcnegocio, $cdtpindice, $idpervalor, $cdindice, $nrpercentindice);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function excluirIndiceFinanceiroPadraoParceiro(Request $request)
    {
        try {
            $params = $request->getParameters();
            $nrindicespadrparc = $params["NRINDICESPADRPARC"];

            $this->businessPartnerService->excluirIndiceFinanceiroPadraoParceiro($nrindicespadrparc);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getSomaPercentualIndicesParceiro(Request $request, Response $response)
    {
        try {
            $params = $request->getParameters();
            $nrparcnegocio     = $params["NRPARCNEGOCIO"];
            $nrindicespadrparc = $params["NRINDICESPADRPARC"];

            $percentual = $this->businessPartnerService->getSomaPercentualIndicesParceiro($nrparcnegocio, $nrindicespadrparc);
            $dataset = new DataSet('retorno', array("percentual" => $percentual));
            $response->addDataSet($dataset);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function validarIndiceDuplicadoParceiro(Request $request)
    {
        try {
            $params = $request->getParameters();
            $nrparcnegocio     = $params["NRPARCNEGOCIO"];
            $nrindicespadrparc = $params["NRINDICESPADRPARC"];
            $cdindice          = $params["CDINDICE"];
            $cdtpindice        = $params["CDTPINDICE"];
            $idpervalor        = $params["IDPERVALOR"];

            $this->businessPartnerService->validarIndiceDuplicadoParceiro($nrparcnegocio, $nrindicespadrparc, $cdindice, $cdtpindice, $idpervalor);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function inserirBairro(Request $request, Response $response)
    {
        try {
            $params = $request->getParameters();
            $cdpais      = $params["CDPAIS"];
            $sgestado    = $params["SGESTADO"];
            $cdmunicipio = $params["CDMUNICIPIO"];
            $nmbairro    = $params["NMBAIRRO"];

            $cdbairro = $this->businessPartnerService->inserirBairro($cdpais, $sgestado, $cdmunicipio, $nmbairro);
            $response->addDataSet(new DataSet("retorno", array("CDBAIRRO" => $cdbairro)));
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getNomeTipoParceiroNegocio(Request $request)
    {
        try {
            $data = $this->businessPartnerService->getNomeTipoParceiroNegocio($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getImportTipoParceiro()
    {
        try {
            $data = $this->businessPartnerService->getImportTipoParceiro();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function importarTipoParceiro(Request $request)
    {
        try {
            $data = $this->businessPartnerService->importarTipoParceiro($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
